==> sunny-foundation-0.2.25.tgz: package/AccessControl/RBAC/PermissionsManager.ts <==
import { isPlainObj } from 'sunny-js/util/object'
import {
  createImplement,
  invokeInterface,
  shouldImplement
} from 'sunny-js/util/module'
import { isEmpty } from 'sunny-js/util/index'

export interface IRBACPrivileges {
  [privilege: string]: any
}

export interface IRBACResource {
  privileges?: IRBACPrivileges
  [key: string]: any
}

export interface IRBACResources {
  [resource: string]: IRBACResource
}

// 定义模块接口
interface ModuleInterfaces {
  ajaxResources: () => Promise<any[]>
  // 用来创建resource key所用的属性名
  resourceProp?: string
  ajaxPrivileges: (resource: IRBACResource) => Promise<any[]>
  // 用来创建Privilege key所用的属性名
  privilegeProp?: string
  // 超级管理员查询接口
  isRootUser?(): boolean
}
const iRBAC = {
  resourceProp: 'path',
  privilegeProp: 'title'
} as ModuleInterfaces
// noinspection JSUnusedGlobalSymbols
export const implement = createImplement(iRBAC)

const ajaxResources = invokeInterface(
  () => iRBAC.ajaxResources,
  shouldImplement('iRBAC.ajaxResources')
)
const ajaxPrivileges = invokeInterface(
  () => iRBAC.ajaxPrivileges,
  shouldImplement('iRBAC.ajaxPrivileges')
)
export const isRootUser = invokeInterface(
  () => iRBAC.isRootUser,
  () => false
)

const _clear = []
// noinspection JSUnusedGlobalSymbols
export function clearCache () {
  _clear.forEach(fn => fn())
}

let _resources: IRBACResources

function cacheResources(): IRBACResources {
  return _resources = _resources || {}
}
_clear.push(() => _resources = null)

function RBACResource (resource: string, data?: object): IRBACResource {
  const cachedPermissions = cacheResources()
  if (resource && isPlainObj(data)) {
    return cachedPermissions[resource] = data
  }
  return cachedPermissions[resource]
}

function RBACPrivileges (resource: string, privileges?: IRBACPrivileges): IRBACPrivileges {
  const cachedPermissions = cacheResources()
  if (resource && isPlainObj(privileges)) {
    privileges = Object.assign(RBACPrivileges(resource) || {}, privileges)
    return cachedPermissions[resource].privileges = privileges
  }
  return cachedPermissions[resource]?.privileges
}

// RBAC权限控制模块所需的resources资源信息，后面的查询都会用到
// noinspection JSUnusedGlobalSymbols
export function getResources () {
  const resources: IRBACResources = cacheResources()
  return !isEmpty(resources)
    ? Promise.resolve(resources)
    : ajaxResources()
      .then(data => {
        data.forEach(item => RBACResource(item[iRBAC.resourceProp], item))
        return cacheResources()
      })
      .catch(err => {
        console.error(err)
        return Promise.reject(err)
      })
}

/**
 * 获取当前资源所拥有的Privileges信息
 * @param resource 默认为当前location.pathname
 */
export function getPrivileges (resource = location.pathname) {
  const privileges = RBACPrivileges(resource)
  if (privileges) {
    return Promise.resolve(privileges)
  }

  const objResource = RBACResource(resource)
  if (isPlainObj(objResource)) { // 有这个资源，就尝试查询该资源有可用privileges
    return ajaxPrivileges(objResource)
      .then(data => {
        const _privileges = {}
        data.forEach(item => _privileges[item[iRBAC.privilegeProp]] = item)
        return RBACPrivileges(resource, _privileges)
      })
      .catch(err => {
        console.error(err)
        return Promise.reject(err)
      })
  }

  // 没有resource默认返回一个空对象
  return Promise.resolve({})
}

// 测试某个Privilege是否存在与resource中
export function hasPrivilege (key: string, resource: string = location.pathname) {
  // 超级管理例外
  if (isRootUser()) {
    return true
  }

  const privileges = RBACPrivileges(resource)
  if (!privileges) {
    console.warn('You should invoke ', getPrivileges, ' to prepare the data ahead for privilege testing')
  }
  return key in (privileges || {})
}

function partialPath (path: string, deep = 2) {
  const hasRootForwardSlash = path.charAt(0) === '/'
  const segments = path.split('/')
  // ['', 'parent', 'child']，截取deep级深度path片段
  let result = segments
    .slice(...[0, deep].map(v => v + (+hasRootForwardSlash)))
    .join('/')
  result = (hasRootForwardSlash ? '/' : '') + result
  return result
}

let _hasResource = {}
// 测试某个resource是否存在
// noinspection JSUnusedGlobalSymbols
export function hasResource (resource: string = location.pathname, deep?: number) {
  // 优先使用缓存
  if (resource in _hasResource) {
    return true
  }

  let result: boolean
  // 父路由可以访问，那么子路由也可以被访问
  const path = partialPath(resource, deep)
  if (path in cacheResources()) { // 直接判断，运气好可以直接通过
    result = true
  } else { // 运气不好，通过遍历检测
    const arrResources = Object.keys(cacheResources())
    result = arrResources.some(_resource => partialPath(_resource, deep) === path)
  }
  // 缓存查询结果
  return _hasResource[resource] = result
}
_clear.push(() => _hasResource = {})

==> sunny-foundation-0.2.27.tgz: package/AccessControl/RBAC/PermissionsManager.ts <==
import { isPlainObj } from 'sunny-js/util/object'
import {
  createImplement,
  invokeInterface,
  shouldImplement
} from 'sunny-js/util/module'
import { isEmpty } from 'sunny-js/util/index'

export interface IRBACPrivileges {
  [privilege: string]: any
}

export interface IRBACResource {
  privileges?: IRBACPrivileges
  [key: string]: any
}

export interface IRBACResources {
  [resource: string]: IRBACResource
}

// 定义模块接口
interface ModuleInterfaces {
  ajaxResources: () => Promise<any[]>
  // 用来创建resource key所用的属性名
  resourceProp?: string
  ajaxPrivileges: (resource: IRBACResource) => Promise<any[]>
  // 用来创建Privilege key所用的属性名
  privilegeProp?: string
  // 超级管理员查询接口
  isRootUser?(): boolean
}
const iRBAC = {
  resourceProp: 'path',
  privilegeProp: 'title'
} as ModuleInterfaces
// noinspection JSUnusedGlobalSymbols
export const implement = createImplement(iRBAC)

const ajaxResources = invokeInterface(
  () => iRBAC.ajaxResources,
  shouldImplement('iRBAC.ajaxResources')
)
const ajaxPrivileges = invokeInterface(
  () => iRBAC.ajaxPrivileges,
  shouldImplement('iRBAC.ajaxPrivileges')
)
export const isRootUser = invokeInterface(
  () => iRBAC.isRootUser,
  () => false
)

const _clear = []
// noinspection JSUnusedGlobalSymbols
export function clearCache () {
  _clear.forEach(fn => fn())
}

let _resources: IRBACResources

function cacheResources(): IRBACResources {
  return _resources = _resources || {}
}
_clear.push(() => _resources = null)

function RBACResource (resource: string, data?: object): IRBACResource {
  const cachedPermissions = cacheResources()
  if (resource && isPlainObj(data)) {
    return cachedPermissions[resource] = data
  }
  return cachedPermissions[resource]
}

function RBACPrivileges (resource: string, privileges?: IRBACPrivileges): IRBACPrivileges {
  const cachedPermissions = cacheResources()
  if (resource && isPlainObj(privileges)) {
    privileges = Object.assign(RBACPrivileges(resource) || {}, privileges)
    return cachedPermissions[resource].privileges = privileges
  }
  return cachedPermissions[resource]?.privileges
}

let updateRequired = false
// noinspection JSUnusedGlobalSymbols
/**
 * 设置一个权限信息更新请求状态，它不是即刻更新，
 * 而是等待下次权限信息查询，才会更新。
 * 也就是说在应用中，如果改动了权限相关信息，告诉当前模块
 * 下次访问权限信息时重新从后端读取最新权限信息
 */
export function requestLazyUpdate () {
  updateRequired = true
}

// RBAC权限控制模块所需的resources资源信息，后面的查询都会用到
// noinspection JSUnusedGlobalSymbols
export function getResources () {
  const resources: IRBACResources = cacheResources()
  // resources资源信息不为空，且不需要更新权限信息时，使用缓存
  return !isEmpty(resources) && !updateRequired
    ? Promise.resolve(resources)
    : ajaxResources()
      .then(data => {
        data.forEach(item => RBACResource(item[iRBAC.resourceProp], item))
        // 更新成功后，updateRequired改为初始状态
        updateRequired = false
        return cacheResources()
      })
      .catch(err => {
        console.error(err)
        return Promise.reject(err)
      })
}

/**
 * 获取当前资源所拥有的Privileges信息
 * @param resource 默认为当前location.pathname
 */
export function getPrivileges (resource = location.pathname) {
  const privileges = RBACPrivileges(resource)
  if (privileges) {
    return Promise.resolve(privileges)
  }

  const objResource = RBACResource(resource)
  if (isPlainObj(objResource)) { // 有这个资源，就尝试查询该资源有可用privileges
    return ajaxPrivileges(objResource)
      .then(data => {
        const _privileges = {}
        data.forEach(item => _privileges[item[iRBAC.privilegeProp]] = item)
        return RBACPrivileges(resource, _privileges)
      })
      .catch(err => {
        console.error(err)
        return Promise.reject(err)
      })
  }

  // 没有resource默认返回一个空对象
  return Promise.resolve({})
}

// 测试某个Privilege是否存在与resource中
export function hasPrivilege (key: string, resource: string = location.pathname) {
  // 超级管理例外
  if (isRootUser()) {
    return true
  }

  const privileges = RBACPrivileges(resource)
  if (!privileges) {
    console.warn('You should invoke ', getPrivileges, ' to prepare the data ahead for privilege testing')
  }
  return key in (privileges || {})
}

function partialPath (path: string, deep = 2) {
  const hasRootForwardSlash = path.charAt(0) === '/'
  const segments = path.split('/')
  // ['', 'parent', 'child']，截取deep级深度path片段
  let result = segments
    .slice(...[0, deep].map(v => v + (+hasRootForwardSlash)))
    .join('/')
  result = (hasRootForwardSlash ? '/' : '') + result
  return result
}

let _hasResource = {}
// 测试某个resource是否存在
// noinspection JSUnusedGlobalSymbols
export function hasResource (resource: string = location.pathname, deep?: number) {
  // 优先使用缓存
  if (resource in _hasResource) {
    return true
  }

  let result: boolean
  // 父路由可以访问，那么子路由也可以被访问
  const path = partialPath(resource, deep)
  if (path in cacheResources()) { // 直接判断，运气好可以直接通过
    result = true
  } else { // 运气不好，通过遍历检测
    const arrResources = Object.keys(cacheResources())
    result = arrResources.some(_resource => partialPath(_resource, deep) === path)
  }
  // 缓存查询结果
  return _hasResource[resource] = result
}
_clear.push(() => _hasResource = {})
